const LOG = require("npm-niosz-log");
const log = LOG.newLog("oracle");
const oracledb = require("oracledb");
if (process.env.LD_LIBRARY_PATH) oracledb.initOracleClient({libDir: process.env.LD_LIBRARY_PATH});
const defaultConfigSize = 1000;
const uuid = require("uuid");
const fs = require("fs");
// ****************************************************************************
const generateFileName = function(i) {
    return i.toString().padStart(10,"0")+".json";
};
// ****************************************************************************
const oraCamelCase = function(column) {
    column = column.toLowerCase();
    column = column.split("_");
    for(var i=1;i<column.length;i++) {
        if (column[i].length>1) {
            column[i] = column[i][0].toUpperCase() + column[i].substring(1);
        } else {
            column[i] = column[i].toUpperCase();
        }
    }
    return column.join("");
};
// ****************************************************************************
const createResultSet = function(folder,fetchs,rows) {
    return {
        folder          : folder,
        fetchs          : fetchs,
        remainingFetchs : fetchs,
        rows            : rows,
        remainingRows   : rows,
        close           : function() { try{fs.rmSync(this.folder,{recursive:true,force:true});return true;}catch{return false;}; },
        fetch           : function(camelCaseOrMapper) {
            try {
                var data = JSON.parse(fs.readFileSync(this.folder+"/"+generateFileName(this.fetchs-this.remainingFetchs),"utf8"));
                this.remainingFetchs--;
                this.remainingRows-=data.length;
                if (camelCaseOrMapper && data.length>0) {
                    if (typeof camelCaseOrMapper == "function") {
                        data = data.map(camelCaseOrMapper);
                    } else {
                        data = data.map((row) => {
                            var camelRow = {};
                            for (prop in row) {
                                const camelProp = oraCamelCase(prop);
                                camelRow[camelProp] = row[prop];
                            }
                            return camelRow;
                        });
                    }
                }
                return data;
            } catch {
                return false;
            }
        }
    };
};
// ****************************************************************************
const oraSelect = async function(connectionConfig,queryConfig) {

    // *** connection to oracle
    var oracle = null;
    try {
        oracle = await oracledb.getConnection(connectionConfig);
        log.i("Connesso correttamente al database Oracle");
    } catch(e) {
        log.e("Connessione in errore:\n"+e);
        return null;
    }

    // *** esecuzione query
    var queryData = null;
    try {
        queryData = await oracle.execute(queryConfig,[],
            {outFormat:oracledb.OUT_FORMAT_OBJECT,prefetchRows:defaultConfigSize,fetchArraySize:defaultConfigSize,resultSet:true});
        log.i("Query eseguita corretamente");
    } catch(e) {
        log.e("Errore nell'esecuzione della query:\n"+e);
        return null;
    }

    // *** prepare folder
    var folderData = null;
    try {
        folderData = "oradata/" + uuid.v4();
        fs.mkdirSync(folderData,{recursive:true});
        log.i("Creata cartella dati : "+folderData);
    } catch(e) {
        log.e("Errore nella creazione cartella dati:\n"+e);
        return null;
    }

    // *** fetching data
    var fetch = 0;
    var len = 0;
    var eof = false;
    while (!eof) {
        var rows = await queryData.resultSet.getRows(defaultConfigSize);
        eof = (!rows.length) ? true : false;
        if (eof) continue;
        fs.writeFileSync(folderData+"/"+generateFileName(fetch),JSON.stringify(rows,null,2),"utf8");
        fetch++;
        len += rows.length;
        log.i("Record processati : " + len);
    }
    log.i("Record totali processati : " + len);

    // *** disconnection from oracle
    try {
        await oracle.close();
        log.i("Disconnesso correttamente dal database Oracle");
    } catch(e) {
        log.w("Disconnesso con segnalazioni dal database Oracle:\n"+e);
    }
    
    // *** create management resultset response
    return createResultSet(folderData,fetch,len);

};
// ****************************************************************************
const ora = {
    select : oraSelect
};
// ****************************************************************************
module.exports = ora;